class PushsController < ApplicationController
  include ActionController::HttpAuthentication::Token
  skip_before_action :verify_authenticity_token

  def token
    param! :app_name, String, required: true
    param! :app_id, String, required: true
    param! :app_secret, String, required: true

    app_setting = Settings.send(params[:app_name].to_sym) rescue {}
    if app_setting['app_id'] == params[:app_id] && app_setting['app_secret'] == params[:app_secret]
      render json: { code: 0, message: 'success', token: Token.generate_token(params[:app_name]), ttl: Token.ttl(params[:app_name])}
    else
      render json: { code: -1, message: '应用不存在或者app_id/app_secret不匹配'}
    end
  end

  # app_name 应用名称
  # user_device_ids 设备ids
  # message 推送信息的json 字符串
  # igetui_opts 个推推送的额外选项
  def push
    param! :app_name, String, required: true
    param! :device_ids_opts, String, required: true
    param! :message, String, required: true
    param! :igetui_opts, String
    param! :app_type, String
    param! :sync_push, String

    igetui_opts = JSON.parse(params[:igetui_opts]) rescue {}
    message = JSON.parse(params[:message]) rescue {}
    device_ids_opts = JSON.parse(params[:device_ids_opts]) rescue {}
    token = token_and_options(request).first
    if token == Token.token(params[:app_name])
      # 写入记录
      NotificationStatistic.create_by_params(igetui_opts, message, device_ids_opts, params[:app_type])

      if params.has_key?(:sync_push)
        timeout_seconds = $redis.get('push_timeout').to_i
        if timeout_seconds > 0
          Timeout.timeout(timeout_seconds) {
            ::PushWorker.new.perform(device_ids_opts, message, igetui_opts, params[:app_type])
          }
        else
          ::PushWorker.new.perform(device_ids_opts, message, igetui_opts, params[:app_type])
        end
      else
        ::PushWorker.perform_async(device_ids_opts, message, igetui_opts, params[:app_type])
      end
      render json: { code: 0, message: 'success', describe: '异步任务正在处理'}
    else
      render json: { code: -1, message: 'token 错误/过期'}
    end
  end

  def received
    param! :notification_id, Integer, required: true
    param! :user_id, Integer, required: true
    param! :app_type, String, required: true
    param! :device_model, String, required: true

    notification_stat = NotificationStatistic.find_by(user_id: params[:user_id], notification_id: params[:notification_id], app_type: params[:app_type])
    notification_stat.update(device_model: params[:device_model], is_received: true) if notification_stat.present?

    render json: { code: 0, message: "success" }
  end

end
