module Push
  module Igetui
    class Push
      attr_accessor :pusher, :pusher_type, :template_type, :template, :template_data

      PUSHERTYPES = %w[push_message_to_single push_message_to_list push_message_to_app].freeze
      TEMPLATETYPES = %w[LinkTemplate NotificationTemplate TransmissionTemplate NotyPopLoadTemplate].freeze
      DEVICEPLATFORMS = %w[android ios].freeze


      def initialize(template_data, opts = {})
        opts = HashWithIndifferentAccess.new(opts)
        @template_data = HashWithIndifferentAccess.new(template_data || {})

        @pusher_type = PUSHERTYPES.include?((opts[:pusher_type]).to_s) ? opts[:pusher_type] : 'push_message_to_single'
        @template_type = TEMPLATETYPES.include?((opts[:template_type]).to_s) ? opts[:template_type] : 'NotificationTemplate'

        ::Push::Log.info("@template_data is #{@template_data.to_json}")
        set_message
        set_template_data
        @message.data = @template
      end

      def push_message_to(client_id)
        client_id_list = Array(client_id)
        case pusher_type
        when 'push_message_to_single'
          client_id_list.map do |client_id|
            client = IGeTui::Client.new(client_id)
            pusher.push_message_to_single(@message, client)
          end
        when 'push_message_to_list'
          content_id = pusher.get_content_id(@message)
          clients = client_id_list.map { |client_id| IGeTui::Client.new(client_id) }

          pusher.push_message_to_list(content_id, clients)
        when 'push_message_to_app'
          @message.app_id_list = client_id_list

          pusher.push_message_to_app(@message)
        end
      end

      private

      def pusher
        @pusher ||=
          if DEVICEPLATFORMS.include?((@template_data['device_platform'].to_s))
            case @template_data['device_platform'].to_s

              when 'android'
                IGeTui.pusher(
                  Settings.platform_settings.igetui.android['app_id'],
                  Settings.platform_settings.igetui.android['app_key'],
                  Settings.platform_settings.igetui.android['master_secret']
                )
              when 'ios'
                IGeTui.pusher(
                  Settings.platform_settings.igetui.ios['app_id'],
                  Settings.platform_settings.igetui.ios['app_key'],
                  Settings.platform_settings.igetui.ios['master_secret']
                )
            end
          else
            ::Push::Log.info("设备类型不正确#{@device_platform}")
            raise "Failed to create pusher"
          end
      end

      def set_message
        @message = case pusher_type
                   when 'push_message_to_single'
                     IGeTui::SingleMessage.new
                   when 'push_message_to_list'
                     IGeTui::ListMessage.new
                   when 'push_message_to_app'
                     IGeTui::AppMessage.new
        end
      end

      def set_template_data
        @template = "IGeTui::#{template_type}".constantize.new
        case template_type
        when 'NotificationTemplate', 'NotyPopLoadTemplate', 'LinkTemplate'
          @template_data.reverse_merge!(Settings.platform_settings.igetui['template_base'])

          @template_data.each do |k, v|
            @template.send("#{k}=", v)
          end
          @template.set_push_info(*@template_data[:push_info]) if @template_data.key? :push_info
        when 'TransmissionTemplate'
          @template.transmission_content = @template_data[:transmission_content].is_a?(Hash) ? @template_data[:transmission_content].to_json : @template_data[:transmission_content]
          @template.transmission_type = @template_data[:transmission_type] if @template_data.key?(:transmission_type)
          @template.set_push_info(*@template_data[:push_info]) if @template_data.key? :push_info
        end
      end
    end
  end
end
